@@ -1,7 +1,7 @@
 /* add.c - ldap BerkeleyDB back-end add routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/add.c,v 1.55.2.6 2002/10/10 01:47:28 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/add.c,v 1.55.2.8 2003/02/09 16:31:37 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -34,7 +34,7 @@ bdb_add(
 #ifdef BDB_SUBENTRIES
 	int subentry;
 #endif
-	u_int32_t	locker;
+	u_int32_t	locker = 0;
 	DB_LOCK		lock;
 #if 0
 	u_int32_t	lockid;
@@ -42,6 +42,10 @@ bdb_add(
 #endif
 	int		noop = 0;
 
+#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+	Operation* ps_list;
+#endif
+
 #ifdef NEW_LOGGING
 	LDAP_LOG ( OPERATION, ARGS, "==> bdb_add: %s\n", e->e_dn, 0, 0 );
 #else
@@ -128,6 +132,7 @@ retry:	/* transaction retry */
 
 	opinfo.boi_bdb = be;
 	opinfo.boi_txn = ltid;
+	opinfo.boi_locker = locker;
 	opinfo.boi_err = 0;
 	op->o_private = &opinfo;
 	
@@ -458,6 +463,7 @@ retry:	/* transaction retry */
 		goto return_results;
 	}
 
+
 	if( op->o_noop ) {
 		if (( rc=TXN_ABORT( ltid )) != 0 ) {
 			text = "txn_abort (no-op) failed";
@@ -542,6 +548,14 @@ return_results:
 	send_ldap_result( conn, op, rc,
 		NULL, text, NULL, NULL );
 
+#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+	if ( rc == LDAP_SUCCESS && !noop ) {
+		LDAP_LIST_FOREACH ( ps_list, &bdb->psearch_list, link ) {
+			bdb_psearch( be, conn, op, ps_list, e, LDAP_PSEARCH_BY_ADD );
+		}
+	}
+#endif /* LDAP_CLIENT_UPDATE */
+
 	if( rc == LDAP_SUCCESS && bdb->bi_txn_cp ) {
 		ldap_pvt_thread_yield();
 		TXN_CHECKPOINT( bdb->bi_dbenv,
@@ -557,3 +571,4 @@ done:
 
 	return ( ( rc == LDAP_SUCCESS ) ? noop : rc );
 }
+
