@@ -1,7 +1,7 @@
 /* init.c - initialize bdb backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/init.c,v 1.75.2.14 2002/11/11 02:42:38 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/init.c,v 1.75.2.18 2003/02/15 15:44:08 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -92,8 +92,10 @@ bdb_db_init( BackendDB *be )
 	bdb->bi_cache.c_maxsize = DEFAULT_CACHE_SIZE;
 
 	bdb->bi_lock_detect = DB_LOCK_DEFAULT;
+	bdb->bi_search_stack_depth = DEFAULT_SEARCH_STACK_DEPTH;
+	bdb->bi_search_stack = NULL;
 
-#ifdef LDAP_CLIENT_UPDATE
+#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
 	LDAP_LIST_INIT (&bdb->psearch_list);
 #endif
 
@@ -106,6 +108,7 @@ bdb_db_init( BackendDB *be )
 #endif
 
 	be->be_private = bdb;
+
 	return 0;
 }
 
@@ -117,20 +120,21 @@ bdb_bt_compare(
 )
 {
 	unsigned char *u, *c;
-	int i;
+	int i, x;
 
 	u = usrkey->data;
 	c = curkey->data;
 
 #ifdef WORDS_BIGENDIAN
-	for( i = 0; i < sizeof(ID); i++)
+	for( i = 0; i < (int)sizeof(ID); i++)
 #else
 	for( i = sizeof(ID)-1; i >= 0; i--)
 #endif
 	{
-		if( u[i] - c[i] )
-			return u[i] - c[i];
+		x = u[i] - c[i];
+		if( x ) return x;
 	}
+
 	return 0;
 }
 
@@ -153,6 +157,18 @@ bdb_db_open( BackendDB *be )
 		be->be_suffix[0].bv_val, 0, 0 );
 #endif
 
+#ifndef BDB_MULTIPLE_SUFFIXES
+	if ( be->be_suffix[1].bv_val ) {
+#ifdef NEW_LOGGING
+	LDAP_LOG( BACK_BDB, ARGS, 
+		"bdb_db_open: only one suffix allowed\n", 0, 0, 0 );
+#else
+	Debug( LDAP_DEBUG_ARGS,
+		"bdb_db_open: only one suffix allowed\n", 0, 0, 0 );
+#endif
+		return -1;
+	}
+#endif
 	/* we should check existance of dbenv_home and db_directory */
 
 	rc = db_env_create( &bdb->bi_dbenv, 0 );
@@ -184,6 +200,13 @@ bdb_db_open( BackendDB *be )
 	bdb->bi_dbenv->set_errcall( bdb->bi_dbenv, bdb_errcall );
 	bdb->bi_dbenv->set_lk_detect( bdb->bi_dbenv, bdb->bi_lock_detect );
 
+#ifdef SLAP_IDL_CACHE
+	if ( bdb->bi_idl_cache_max_size ) {
+		ldap_pvt_thread_mutex_init( &bdb->bi_idl_tree_mutex );
+		bdb->bi_idl_cache_size = 0;
+	}
+#endif
+
 #ifdef BDB_SUBDIRS
 	{
 		char dir[MAXPATHLEN];
@@ -402,6 +425,9 @@ bdb_db_close( BackendDB *be )
 	int rc;
 	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
 	struct bdb_db_info *db;
+#ifdef SLAP_IDL_CACHE
+	bdb_idl_cache_entry_t *entry, *next_entry;
+#endif
 
 	while( bdb->bi_ndatabases-- ) {
 		db = bdb->bi_databases[bdb->bi_ndatabases];
@@ -416,6 +442,19 @@ bdb_db_close( BackendDB *be )
 
 	bdb_cache_release_all (&bdb->bi_cache);
 
+#ifdef SLAP_IDL_CACHE
+	ldap_pvt_thread_mutex_lock ( &bdb->bi_idl_tree_mutex );
+	entry = bdb->bi_idl_lru_head;
+	while ( entry != NULL ) {
+		next_entry = entry->idl_lru_next;
+		free( entry->idl );
+		free( entry->kstr.bv_val );
+		free( entry );
+		entry = next_entry;
+	}
+	ldap_pvt_thread_mutex_unlock ( &bdb->bi_idl_tree_mutex );
+#endif
+
 	return 0;
 }
 
@@ -495,15 +534,14 @@ bdb_initialize(
 {
 	static char *controls[] = {
 		LDAP_CONTROL_MANAGEDSAIT,
+		LDAP_CONTROL_NOOP,
+#ifdef LDAP_CONTROL_PAGEDRESULTS
+		LDAP_CONTROL_PAGEDRESULTS,
+#endif
+ 		LDAP_CONTROL_VALUESRETURNFILTER,
 #ifdef LDAP_CONTROL_SUBENTRIES
 		LDAP_CONTROL_SUBENTRIES,
 #endif
-#ifdef LDAP_CONTROL_NOOP
-		LDAP_CONTROL_NOOP,
-#endif
-#ifdef LDAP_CONTROL_VALUESRETURNFILTER
- 		LDAP_CONTROL_VALUESRETURNFILTER,
-#endif
 #ifdef LDAP_CLIENT_UPDATE
 		LDAP_CONTROL_CLIENT_UPDATE,
 #endif
@@ -597,7 +635,14 @@ bdb_initialize(
 	bi->bi_op_search = bdb_search;
 
 	bi->bi_op_unbind = 0;
+
+#ifdef LDAP_CLIENT_UPDATE
+	bi->bi_op_abandon = bdb_abandon;
+	bi->bi_op_cancel = bdb_cancel;
+#else
 	bi->bi_op_abandon = 0;
+	bi->bi_op_cancel = 0;
+#endif
 
 	bi->bi_extended = bdb_extended;
 
@@ -615,9 +660,7 @@ bdb_initialize(
 
 	bi->bi_chk_referrals = bdb_referrals;
 	bi->bi_operational = bdb_operational;
-#ifdef SLAP_X_FILTER_HASSUBORDINATES
 	bi->bi_has_subordinates = bdb_hasSubordinates;
-#endif /* SLAP_X_FILTER_HASSUBORDINATES */
 	bi->bi_entry_release_rw = bdb_entry_release;
 
 	/*
