@@ -1,7 +1,7 @@
 /* modify.c - bdb backend modify routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/modify.c,v 1.44.2.7 2002/09/01 04:00:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/modify.c,v 1.44.2.10 2003/02/09 17:41:28 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -56,7 +56,8 @@ int bdb_modify_internal(
 #else
 			Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: add\n", 0, 0, 0);
 #endif
-			err = modify_add_values( e, mod, text, textbuf, textlen );
+			err = modify_add_values( e, mod, get_permissiveModify(op),
+				text, textbuf, textlen );
 			if( err != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
 				LDAP_LOG ( OPERATION, ERR, 
@@ -75,7 +76,8 @@ int bdb_modify_internal(
 #else
 			Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: delete\n", 0, 0, 0);
 #endif
-			err = modify_delete_values( e, mod, text, textbuf, textlen );
+			err = modify_delete_values( e, mod, get_permissiveModify(op),
+				text, textbuf, textlen );
 			assert( err != LDAP_TYPE_OR_VALUE_EXISTS );
 			if( err != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
@@ -95,7 +97,8 @@ int bdb_modify_internal(
 #else
 			Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: replace\n", 0, 0, 0);
 #endif
-			err = modify_replace_values( e, mod, text, textbuf, textlen );
+			err = modify_replace_values( e, mod, get_permissiveModify(op),
+				text, textbuf, textlen );
 			if( err != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
 				LDAP_LOG ( OPERATION, ERR, 
@@ -119,7 +122,8 @@ int bdb_modify_internal(
  			 */
  			mod->sm_op = LDAP_MOD_ADD;
 
-			err = modify_add_values( e, mod, text, textbuf, textlen );
+			err = modify_add_values( e, mod, get_permissiveModify(op),
+				text, textbuf, textlen );
  			if ( err == LDAP_TYPE_OR_VALUE_EXISTS ) {
  				err = LDAP_SUCCESS;
  			}
@@ -272,11 +276,17 @@ bdb_modify(
 	DB_TXN	*ltid = NULL;
 	struct bdb_op_info opinfo;
 
-	u_int32_t	locker;
+	u_int32_t	locker = 0;
 	DB_LOCK		lock;
 
 	int		noop = 0;
 
+#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+	Operation* ps_list;
+	struct psid_entry* pm_list;
+	struct psid_entry* pm_prev;
+#endif
+
 #ifdef NEW_LOGGING
 	LDAP_LOG ( OPERATION, ENTRY, "bdb_modify: %s\n", dn->bv_val, 0, 0 );
 #else
@@ -295,6 +305,17 @@ retry:	/* transaction retry */
 		Debug(LDAP_DEBUG_TRACE,
 			"bdb_modify: retrying...\n", 0, 0, 0);
 #endif
+
+#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+		pm_list = LDAP_LIST_FIRST(&op->premodify_list);
+		while ( pm_list != NULL ) {
+			LDAP_LIST_REMOVE ( pm_list, link );
+			pm_prev = pm_list;
+			pm_list = LDAP_LIST_NEXT ( pm_list, link );
+			free (pm_prev);
+		}
+#endif
+
 		rc = TXN_ABORT( ltid );
 		ltid = NULL;
 		op->o_private = NULL;
@@ -328,6 +349,7 @@ retry:	/* transaction retry */
 
 	opinfo.boi_bdb = be;
 	opinfo.boi_txn = ltid;
+	opinfo.boi_locker = locker;
 	opinfo.boi_err = 0;
 	op->o_private = &opinfo;
 
@@ -405,6 +427,14 @@ retry:	/* transaction retry */
 		ber_bvarray_free( refs );
 		goto done;
 	}
+
+#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+	if ( rc == LDAP_SUCCESS && !op->o_noop ) {
+		LDAP_LIST_FOREACH ( ps_list, &bdb->psearch_list, link ) {
+			bdb_psearch(be, conn, op, ps_list, e, LDAP_PSEARCH_BY_PREMODIFY );
+		}
+	}
+#endif
 	
 	/* Modify the entry */
 	rc = bdb_modify_internal( be, conn, op, ltid, modlist, e,
@@ -493,6 +523,24 @@ return_results:
 	send_ldap_result( conn, op, rc,
 		NULL, text, NULL, NULL );
 
+#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+	if ( rc == LDAP_SUCCESS && !op->o_noop ) {
+		/* Loop through in-scope entries for each psearch spec */
+		LDAP_LIST_FOREACH ( ps_list, &bdb->psearch_list, link ) {
+			bdb_psearch( be, conn, op, ps_list, e, LDAP_PSEARCH_BY_MODIFY );
+		}
+		pm_list = LDAP_LIST_FIRST(&op->premodify_list);
+		while ( pm_list != NULL ) {
+			bdb_psearch(be, conn, op, pm_list->ps->op,
+						e, LDAP_PSEARCH_BY_SCOPEOUT);
+			LDAP_LIST_REMOVE ( pm_list, link );
+			pm_prev = pm_list;
+			pm_list = LDAP_LIST_NEXT ( pm_list, link );
+			free (pm_prev);
+		}
+	}
+#endif
+
 	if( rc == LDAP_SUCCESS && bdb->bi_txn_cp ) {
 		ldap_pvt_thread_yield();
 		TXN_CHECKPOINT( bdb->bi_dbenv,
@@ -501,6 +549,15 @@ return_results:
 
 done:
 	if( ltid != NULL ) {
+#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+		pm_list = LDAP_LIST_FIRST(&op->premodify_list);
+		while ( pm_list != NULL ) {
+			LDAP_LIST_REMOVE ( pm_list, link );
+			pm_prev = pm_list;
+			pm_list = LDAP_LIST_NEXT ( pm_list, link );
+			free (pm_prev);
+		}
+#endif
 		TXN_ABORT( ltid );
 		op->o_private = NULL;
 	}
