@@ -95,10 +95,10 @@ monitor_subsys_log_init(
 	if ( monitor_cache_get( mi, &monitor_subsys[SLAPD_MONITOR_LOG].mss_ndn, 
 				&e ) ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
+		LDAP_LOG( OPERATION, CRIT,
 			"monitor_subsys_log_init: "
 			"unable to get entry '%s'\n",
-			monitor_subsys[SLAPD_MONITOR_LOG].mss_ndn.bv_val ));
+			monitor_subsys[SLAPD_MONITOR_LOG].mss_ndn.bv_val, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_log_init: "
@@ -180,7 +180,7 @@ monitor_subsys_log_modify(
 			break;
 
 		default:
-			rc = LDAP_OPERATIONS_ERROR;
+			rc = LDAP_OTHER;
 			break;
 		}
 
