@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/main.c,v 1.132.2.8 2002/11/11 02:42:37 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/main.c,v 1.132.2.11 2003/02/21 19:58:46 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 #include "portable.h"
@@ -20,6 +20,10 @@
 #include "lutil.h"
 #include "ldif.h"
 
+#ifdef LDAP_SLAPI
+#include "slapi.h"
+#endif
+
 #ifdef LDAP_SIGCHLD
 static RETSIGTYPE wait4child( int sig );
 #endif
@@ -81,7 +85,7 @@ static STRDISP	syslog_types[] = {
 	{ "LOCAL5", sizeof("LOCAL5"), LOG_LOCAL5 },
 	{ "LOCAL6", sizeof("LOCAL6"), LOG_LOCAL6 },
 	{ "LOCAL7", sizeof("LOCAL7"), LOG_LOCAL7 },
-	{ NULL }
+	{ NULL, 0, 0 }
 };
 
 static int   cnvt_str2int( char *, STRDISP_P, int );
@@ -157,7 +161,6 @@ int main( int argc, char **argv )
 	}
 #endif
 
-
 #ifdef HAVE_NT_SERVICE_MANAGER
 	{
 		int *i;
@@ -325,7 +328,6 @@ int main( int argc, char **argv )
 	Debug( LDAP_DEBUG_TRACE, "%s", Versionstr, 0, 0 );
 #endif
 
-
 	if( serverName == NULL ) {
 		if ( (serverName = strrchr( argv[0], *LDAP_DIRSEP )) == NULL ) {
 			serverName = argv[0];
@@ -405,6 +407,20 @@ int main( int argc, char **argv )
 	(void) ldap_pvt_tls_set_option( NULL, LDAP_OPT_X_TLS_REQUIRE_CERT, &rc );
 #endif
 
+#ifdef LDAP_SLAPI
+	if ( slapi_init() != 0 ) {
+#ifdef NEW_LOGGING
+		LDAP_LOG( OPERATION, CRIT, "main: slapi initialization error\n", 0, 0, 0 );
+#else
+		Debug( LDAP_DEBUG_ANY,
+		    "slapi initialization error\n",
+		    0, 0, 0 );
+#endif
+
+		goto destroy;
+	}
+#endif /* LDAP_SLAPI */
+
 	if ( read_config( configfile, 0 ) != 0 ) {
 		rc = 1;
 		SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 19 );
@@ -508,31 +524,38 @@ int main( int argc, char **argv )
 		goto shutdown;
 	}
 
-	{
-		FILE *fp;
-
 #ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, INFO, "main: slapd starting.\n", 0, 0, 0 );
+	LDAP_LOG( SLAPD, INFO, "main: slapd starting.\n", 0, 0, 0 );
 #else
-		Debug( LDAP_DEBUG_ANY, "slapd starting\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_ANY, "slapd starting\n", 0, 0, 0 );
 #endif
 
 
-		if (( slapd_pid_file != NULL ) &&
-			(( fp = fopen( slapd_pid_file, "w" )) != NULL ))
-		{
+	if ( slapd_pid_file != NULL ) {
+		FILE *fp = fopen( slapd_pid_file, "w" );
+
+		if( fp != NULL ) {
 			fprintf( fp, "%d\n", (int) getpid() );
 			fclose( fp );
-		}
 
-		if (( slapd_args_file != NULL ) &&
-			(( fp = fopen( slapd_args_file, "w" )) != NULL ))
-		{
+		} else {
+			free(slapd_pid_file);
+			slapd_pid_file = NULL;
+		}
+	}
+
+	if ( slapd_args_file != NULL ) {
+		FILE *fp = fopen( slapd_args_file, "w" );
+
+		if( fp != NULL ) {
 			for ( i = 0; i < g_argc; i++ ) {
 				fprintf( fp, "%s ", g_argv[i] );
 			}
 			fprintf( fp, "\n" );
 			fclose( fp );
+		} else {
+			free(slapd_args_file);
+			slapd_args_file = NULL;
 		}
 	}
 
@@ -591,6 +614,13 @@ stop:
 	ldap_pvt_tls_destroy();
 #endif
 
+	if ( slapd_pid_file != NULL ) {
+		unlink( slapd_pid_file );
+	}
+	if ( slapd_args_file != NULL ) {
+		unlink( slapd_args_file );
+	}
+
 	config_destroy();
 
 #ifdef CSRIMALLOC
